use tokio::net::TcpStream;
use tokio_socks::tcp::Socks5Stream;

use crate::messenger::error::MessengerError;

/// Parameters used in establishing the connection, optionally through a socks proxy.
/// (Members may be useful elsewhere as well, but that's the primary purpose.)
pub struct SocksParams {
    /// Optional socks proxy address.
    pub socks: Option<String>,
    /// The target server or peer.
    pub target: String,
    /// The user who owns this connection.
    pub user: String,
    /// The recipient of messages sent on this connection.
    /// Group for client-server, user for p2p.
    pub recipient: String,
}

pub async fn connect(str_params: &SocksParams) -> Result<TcpStream, MessengerError> {
    match &str_params.socks {
        Some(socks) => {
            let socks_connection = Socks5Stream::connect_with_password(
                socks.as_str(),
                str_params.target.as_str(),
                &str_params.user,
                &str_params.recipient,
            )
            .await;
            match socks_connection {
                Ok(stream) => Ok(stream.into_inner()),
                Err(e) => Err(e.into()),
            }
        }
        None => TcpStream::connect(&str_params.target)
            .await
            .map_err(|e| e.into()),
    }
}
